"""Result dataclasses for forecasts and backtests."""

from timesmith.results.forecast import Forecast
from timesmith.results.backtest import BacktestResult

__all__ = [
    "Forecast",
    "BacktestResult",
]

